@@ -1,4 +1,3 @@
-
 parse_remote_param <- function(specs, config, ...) {
   parsed_specs <- re_match(specs, param_rx())
   parsed_specs$ref <- "*"
@@ -7,18 +6,31 @@ parse_remote_param <- function(specs, config, ...) {
   parsed_specs$type <- "param"
   lapply(
     seq_len(nrow(parsed_specs)),
-    function(i) as.list(parsed_specs[i,])
+    function(i) as.list(parsed_specs[i, ])
   )
 }
 
-resolve_remote_param <- function(remote, direct, config, cache,
-                                 dependencies, ...) {
+resolve_remote_param <- function(
+  remote,
+  direct,
+  config,
+  cache,
+  dependencies,
+  ...
+) {
   ## Handled specially by the resolver
   NULL
 }
 
-download_remote_param <- function(resolution, target, target_tree, config,
-                                  cache, which, on_progress) {
+download_remote_param <- function(
+  resolution,
+  target,
+  target_tree,
+  config,
+  cache,
+  which,
+  on_progress
+) {
   ## this is never actually called
   stop("Internal error")
 }
