@@ -1,10 +1,9 @@
-
 test_that("merge_new", {
   expect_identical(merge_new("x", "y"), "y")
   expect_identical(merge_new("x", "y", "replace"), "y")
   expect_identical(merge_new("x", "y", "prepend"), c("y", "x"))
   expect_identical(merge_new("x", "y", "append"), c("x", "y"))
-  expect_error(merge_new("x", "y", "foobar"))
+  expect_snapshot(error = TRUE, merge_new("x", "y", "foobar"))
 
   ## Some special values
   expect_identical(merge_new("x", NULL), NULL)
