@@ -1,4 +1,3 @@
-
 (function() {
   trace <- rev(sys.calls())
   source_arg <- NULL
@@ -26,11 +25,14 @@
 
   message("Installing pak from stream ", stream, ".")
 
-  install.packages("pak", repos = sprintf(
-    "https://r-lib.github.io/p/pak/%s/%s/%s/%s",
-    stream,
-    .Platform$pkgType,
-    R.Version()$os,
-    R.Version()$arch
-  ))
+  install.packages(
+    "pak",
+    repos = sprintf(
+      "https://r-lib.github.io/p/pak/%s/%s/%s/%s",
+      stream,
+      .Platform$pkgType,
+      R.Version()$os,
+      R.Version()$arch
+    )
+  )
 })()
